package de.mdelab.mlsdm.interpreter.debug.protocol.indications;

import org.eclipse.net4j.signal.SignalProtocol;
import org.eclipse.net4j.util.io.ExtendedDataInputStream;
import org.eclipse.net4j.util.io.ExtendedDataOutputStream;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlexpressions.MLStringExpression;
import de.mdelab.mlexpressions.MlexpressionsFactory;
import de.mdelab.sdm.interpreter.core.debug.SDDebugger;
import de.mdelab.sdm.interpreter.core.debug.breakpoints.Breakpoint;
import de.mdelab.sdm.interpreter.core.debug.breakpoints.ConditionalBreakpoint;
import de.mdelab.sdm.interpreter.core.debug.protocol.SDDebugProtocolCommandMessagesEnum;
import de.mdelab.sdm.interpreter.core.debug.protocol.indications.SDDebugProtocolCommandIndicationWithResponse;

public class SetBreakpointCommandIndication extends SDDebugProtocolCommandIndicationWithResponse<MLElementWithUUID>
{

	private Breakpoint<MLElementWithUUID>								breakpoint;

	private final SDDebugger<MLElementWithUUID, ?, ?, ?, MLExpression>	debugger;

	public SetBreakpointCommandIndication(final SignalProtocol<?> protocol,
			final SDDebugger<MLElementWithUUID, ?, ?, ?, MLExpression> debugger)
	{
		super(protocol, SDDebugProtocolCommandMessagesEnum.SET_BREAKPOINT, debugger);

		this.debugger = debugger;
	}

	@Override
	protected void indicating(final ExtendedDataInputStream in) throws Exception
	{
		/*
		 * Read elementUUID
		 */
		final String elementUUID = (String) in.readObject();

		/*
		 * Read expression language
		 */
		final String expressionLanguage = (String) in.readObject();

		/*
		 * Read expression string
		 */
		final String expressionString = (String) in.readObject();

		/*
		 * Set breakpoint
		 */
		MLStringExpression expression = null;

		if ((expressionString != null) && !"".equals(expressionString))
		{
			assert expressionLanguage != null;
			assert !"".equals(expressionLanguage);

			expression = MlexpressionsFactory.eINSTANCE.createMLStringExpression();
			expression.setExpressionLanguageID(expressionLanguage);
			expression.setExpressionString(expressionString);
		}

		this.breakpoint = new ConditionalBreakpoint<MLElementWithUUID, MLExpression>(this.debugger, elementUUID, expression);

		this.getDebugger().addBreakpoint(this.breakpoint);
	}

	@Override
	protected void responding(final ExtendedDataOutputStream out) throws Exception
	{
		out.writeString(this.breakpoint.getBreakpointUUID());
	}

}
