package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints;

import org.eclipse.core.resources.IMarkerDelta;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.IBreakpointListener;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.debug.internal.ui.views.breakpoints.BreakpointsLabelProvider;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.AbstractDecorator;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoration;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorTarget;
import org.eclipse.gmf.runtime.draw2d.ui.mapmode.IMapMode;
import org.eclipse.gmf.runtime.draw2d.ui.mapmode.MapModeUtil;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.graphics.Image;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.ActivityFinalNode;
import de.mdelab.mlsdm.DecisionMergeNode;
import de.mdelab.mlsdm.FlowFinalNode;
import de.mdelab.mlsdm.InitialNode;
import de.mdelab.mlstorypatterns.StoryPatternLink;
import de.mdelab.mlstorypatterns.StoryPatternObject;
import de.mdelab.mlstorypatterns.diagram.custom.edit.parts.IMLExpressionEditPart;
import de.mdelab.sdm.interpreter.code.debug.ui.breakpoints.SDDebugBreakpoint;

@SuppressWarnings("restriction")
public class BreakpointDecorator extends AbstractDecorator implements IBreakpointListener /*implements IBreakpointsManagerListener*/ {

	//private boolean	subscribed = false;

	//private String	uuid;
	
	private final ILabelProvider labelProvider;

	public BreakpointDecorator( final IDecoratorTarget decoratorTarget ) {
		super( decoratorTarget );
		
		labelProvider = new BreakpointsLabelProvider();
		
		DebugPlugin.getDefault().getBreakpointManager().addBreakpointListener( this );
	}

	@Override
	public void activate() {
		refresh();
	}
	
	private String getUuidForBreakpoints() {
		final MLElementWithUUID elem = getModelElement();
		
		return elem == null ? null : elem.getUuid();
	}

//	private String getUuidForBreakpoints( final MLElementWithUUID p_element ) {
//		final EditPart editPart = getEditPart();
//		
//		String uuid = null;
//		
//		if ( editPart instanceof WrappingLabel4EditPart ) {
//			if ( ( (ActivityEdge) p_element ).getGuardExpression() != null)	{
//				uuid = ( (ActivityEdge) p_element ).getGuardExpression().getUuid();
//			}
//		}
//		else if ( editPart instanceof StoryPatternStoryPatternConstraintsCompartmentEditPart ) {
//			if ( !( (StoryNode) p_element ).getStoryPattern().getConstraints().isEmpty() ) {
//				uuid = ( (StoryNode) p_element ).getStoryPattern().getConstraints().get( 0 ).getUuid();
//			}
//		}
//		else if ( editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart ) {
//			if ( !( (StoryPatternObject) p_element ).getConstraints().isEmpty()) {
//				uuid = ( (StoryPatternObject) p_element ).getConstraints().get( 0 ).getUuid();
//			}
//		}
//		else if ( editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart ) {
//			if ( !( (StoryPatternObject) p_element ).getAttributeAssignments().isEmpty() ) {
//				uuid = ( (StoryPatternObject) p_element ).getAttributeAssignments().get( 0 ).getUuid();
//			}
//		}
//		else {
//			uuid = p_element.getUuid();
//		}
//		
//		return uuid;
//	}
	
	private EditPart getEditPart() {
		return (EditPart) this.getDecoratorTarget().getAdapter( EditPart.class );
	}
	
	@Override
	public void refresh() {
		this.removeDecoration();

		//final EditPart editPart = (EditPart) this.getDecoratorTarget().getAdapter( EditPart.class );
//
//		final View view = (View) this.getDecoratorTarget().getAdapter(View.class);
//
//		if (!(view.getElement() instanceof MLElementWithUUID) || (view == null))
//		{
//			return;
//		}

		final String uuid = getUuidForBreakpoints();
//
//		if (editPart instanceof WrappingLabel4EditPart/* ActivityEdgeGuardConstraintLabelEditPart */)
//		{
//			if (((ActivityEdge) element).getGuardExpression() == null)
//			{
//				return;
//			}
//			uuid = ((ActivityEdge) element).getGuardExpression().getUuid();
//		}
//		else if (editPart instanceof StoryPatternStoryPatternConstraintsCompartmentEditPart)
//		{
//			if (((StoryNode) element).getStoryPattern().getConstraints().isEmpty())
//			{
//				return;
//			}
//			uuid = ((StoryNode) element).getStoryPattern().getConstraints().get(0).getUuid();
//		}
//		else if (editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart)
//		{
//			if (((StoryPatternObject) element).getConstraints().isEmpty())
//			{
//				return;
//			}
//			uuid = ((StoryPatternObject) element).getConstraints().get(0).getUuid();
//		}
//		else if (editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart)
//		{
//			if (((StoryPatternObject) element).getAttributeAssignments().isEmpty())
//			{
//				return;
//			}
//			uuid = ((StoryPatternObject) element).getAttributeAssignments().get(0).getUuid();
//		}
//		else
//		{
//			uuid = element.getUuid();
//		}

//		final IBreakpoint[] breakpoints = DebugPlugin.getDefault().getBreakpointManager()
//				.getBreakpoints(SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER);

		// look if breakpoint is intended for my decorator target
		//IBreakpoint breakpoint = null;

		if ( uuid != null ) {
			final IBreakpoint breakpoint = SDDebugBreakpoint.findBreakpoint( uuid );

			if ( breakpoint != null ) {
				doUpdateDecoration( breakpoint );
			}
		}

//		if (breakpoint != null)
//		{
//			URL url = null;
//			final String PLUGIN_ID = "de.mdelab.mlsdm.interpreter.debug.ui";
//			final Bundle bundle = Platform.getBundle(/* SDDebugUiPlugin. */PLUGIN_ID);
//
//			try
//			{
//				if (!this.subscribed)
//				{
//					//MLSDMBreakpointsManager.getModelProvider().addListener(this);
//					this.subscribed = true;
//				}

				//checkBreakpointLocation( view.eResource().getURI().toString(), (SDDebugBreakpoint) breakpoint );
//
//				if (breakpoint.isEnabled())
//				{
//					url = bundle.getEntry("icons/enabled/breakpoint.gif");
//				}
//				else
//				{
//					url = bundle.getEntry("icons/disabled/breakpoint.gif");
//				}
//			}
//			catch (final CoreException e)
//			{
//				e.printStackTrace();
//			}

//			final ImageDescriptor imageDesc = ImageDescriptor.createFromURL(url);
//			final Image image = imageDesc.createImage();
//			if ((editPart instanceof CustomActivityEdgeGuardConstraintLabelEditPart)
//					|| (editPart instanceof StoryNodeStoryNodeElementsCompartmentEditPart)
//					|| (editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart)
//					|| (editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart))
//			{
//				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.WEST,
//						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(0), false));
//			}
//			else if ((element instanceof ActivityEdge) || (element instanceof StoryPatternLink))
//			{
//				this.setDecoration(this.getDecoratorTarget().addConnectionDecoration(image, 50, false));
//
//			}
//			else if ((element instanceof InitialNode) || (element instanceof FlowFinalNode) || (element instanceof ActivityFinalNode))
//			{
//				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.SOUTH_WEST,
//						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(-8), false));
//			}
//			else if ((element instanceof DecisionMergeNode) || (element instanceof InitialNode) || (element instanceof FlowFinalNode)
//					|| (element instanceof ActivityFinalNode))
//			{
//				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.CENTER,
//						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(-4), false));
//
//			}
//			else if (element instanceof StoryPatternObject)
//			{
//				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.NORTH_EAST, -1, false));
//			}
//			else if (element instanceof MLStringExpression)
//			{
//				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.EAST,
//						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(0), false));
//			}
//			else
//			{
//				this.setDecoration(this.getDecoratorTarget().addShapeDecoration(image, IDecoratorTarget.Direction.NORTH_EAST,
//						MapModeUtil.getMapMode(((AbstractGraphicalEditPart) editPart).getFigure()).DPtoLP(-4), false));
//			}
//		}
//		else
//		{
//			if (this.subscribed)
//			{
				//MLSDMBreakpointsManager.getModelProvider().removeListener(this);
//				this.subscribed = false;
//				uuid = null;
//			}
//		}
	}
//
//	@Override
//	public void breakpointChanged(final BreakpointsManagerEvent event)
//	{
//
//		if (event.getItem().getUuid().equals(this.uuid))
//		{
//			final View view = (View) this.getDecoratorTarget().getAdapter(View.class);
//			try
//			{
//				final String string = view.eResource().getURI().toString();
//				if (string != null)
//				{
//					this.refresh();
//				}
//			}
//			catch (final Exception e)
//			{
//				MLSDMBreakpointsManager.getModelProvider().removeListener(this);
//			}
//		}
//	}

	private MLElementWithUUID getModelElement() {
		final EditPart editPart = getEditPart();
		
		if ( editPart instanceof IMLExpressionEditPart ) {
			return ((IMLExpressionEditPart) editPart ).getMLExpression();
		}
		
		return (MLElementWithUUID) editPart.getAdapter( MLElementWithUUID.class );
	}
	
	private void doUpdateDecoration( final IBreakpoint p_breakpoint ) {
		final MLElementWithUUID element = getModelElement();
		
		assert element != null;
		
		final Image image = labelProvider.getImage( p_breakpoint );
		
		final GraphicalEditPart editPart = (GraphicalEditPart) getDecoratorTarget().getAdapter( GraphicalEditPart.class );
		
		final IMapMode mapMode = MapModeUtil.getMapMode( editPart.getFigure() );

		removeDecoration();
		
		final IDecoration decoration;

		/*if (	editPart instanceof CustomActivityEdgeGuardConstraintLabelEditPart || 
				editPart instanceof StoryNodeStoryNodeElementsCompartmentEditPart ||
				editPart instanceof StoryPatternObjectStoryPatternObjectConstraintsCompartmentEditPart ||
				editPart instanceof StoryPatternObjectStoryPatternObjectAttributeAssignmentsCompartmentEditPart ) {
			decoration = getDecoratorTarget().addShapeDecoration( 	image, 
																	IDecoratorTarget.Direction.WEST,
																	mapMode.DPtoLP( 0 ),
																	false );
		}
		else*/ if ( element instanceof ActivityEdge || element instanceof StoryPatternLink ) {
			decoration = getDecoratorTarget().addConnectionDecoration( image, 50, false );

		}
		else if ( element instanceof InitialNode || element instanceof FlowFinalNode || element instanceof ActivityFinalNode ) {
			decoration = getDecoratorTarget().addShapeDecoration( 	image,
																	IDecoratorTarget.Direction.SOUTH_WEST,
																	mapMode.DPtoLP(-8),
																	false );
		}
		else if ( element instanceof DecisionMergeNode ) {
			decoration = getDecoratorTarget().addShapeDecoration( 	image,
																	IDecoratorTarget.Direction.CENTER,
																	mapMode.DPtoLP( -4 ), 
																	false );

		}
		else if ( element instanceof StoryPatternObject ) {
			decoration = getDecoratorTarget().addShapeDecoration( image, IDecoratorTarget.Direction.NORTH_EAST, -1, false );
		}
		else if ( element instanceof MLExpression ) {
			decoration = getDecoratorTarget().addShapeDecoration(	image,
																	IDecoratorTarget.Direction.CENTER,
																	mapMode.DPtoLP( 0 ),
																	false );
		}
		else {
			decoration = getDecoratorTarget().addShapeDecoration(	image,
																	IDecoratorTarget.Direction.NORTH_EAST,
																	mapMode.DPtoLP( -4 ),
																	false );
		}
		
		setDecoration( decoration );
	}
	
	private boolean isConcerned( final IBreakpoint p_breakpoint ) {
		if ( p_breakpoint instanceof SDDebugBreakpoint ) {
			final String uuid = getUuidForBreakpoints();
			
			return uuid != null && uuid.equals( ( (SDDebugBreakpoint) p_breakpoint ).getElementUUID()  );
		}
		
		return false;
	}

	@Override
	public void breakpointAdded( final IBreakpoint p_breakpoint ) {
		if ( isConcerned( p_breakpoint ) ) {
			doUpdateDecoration( p_breakpoint );
		}
	}

	@Override
	public void breakpointRemoved(	final IBreakpoint p_breakpoint,
									final IMarkerDelta delta ) {
		if ( isConcerned( p_breakpoint ) ) {
			removeDecoration();
		}
	}

	@Override
	public void breakpointChanged(	final IBreakpoint p_breakpoint, 
									final IMarkerDelta p_delta ) {
		if ( isConcerned( p_breakpoint ) ) {
			doUpdateDecoration( p_breakpoint );
		}
	}
	
	@Override
	public void finalize() 
	throws Throwable {
		DebugPlugin.getDefault().getBreakpointManager().removeBreakpointListener( this );
		labelProvider.dispose();
		
		super.finalize();
	}
}
